local addonName, addonData = ...

addonData.chatWindowsReady = false

--[[ ]]--
function addonData:FindAvailableChatFrame(chatData)
    local retValue = nil
    
    -- first see if the one saved is safe to be used still 
    if chatData and chatData.ID then
        local name, fontSize, red, green, blue, alpha, shown, locked, docked, uninteractable = FCF_GetChatWindowInfo(chatData.ID)
        local chatFrame = Chat_GetChatFrame(chatData.ID)
        local infoPanel = chatFrame.infoPanel        
        local chatKey = "ChatFrame"..chatData.ID
        if not shown and not docked and not infoPanel then             
            return chatKey
        end
    end
    
    -- Nope, then cycle through frames to find one available
    for i = 1,NUM_CHAT_WINDOWS do
        local name, fontSize, red, green, blue, alpha, shown, locked, docked, uninteractable = FCF_GetChatWindowInfo(i)
        local chatFrame = Chat_GetChatFrame(i)
        local infoPanel = chatFrame.infoPanel
        
        local chatKey = "ChatFrame"..i
        
        if not shown and not docked and not infoPanel then 
            retValue = chatKey
            break
        end

    end
    return retValue
end

--[[ ]]--
function addonData:GetSavedVariablePath()
    
    -- Is it Account/None, Account/Character or Character/None Wide Data ?
    local svPath = self:SVPath(self.SVType.Character, self.SVKey.None)
    
    -- Grab the "ChatFrame" subtable from the selected path
    local chatData = self:SVData("ChatFrame",self.SVMode.RetrieveSubTable,svPath)
    
    -- return the resulting table of data
    return chatData
end

--[[ ]]--
function addonData:SaveChatFrameInfo(chatFrame)
    local id = chatFrame:GetID()
    local name, fontSize, red, green, blue, alpha, shown, locked, docked, uninteractable = FCF_GetChatWindowInfo(id)
    local chatFrame = Chat_GetChatFrame(id)
    local fontFile,fontHeight,fontFlags = chatFrame:GetFont()
    local minimized = chatFrame.minimized
    local channelList = chatFrame.channelList
    local zoneChannelList = chatFrame.zoneChannelList
    local messageTypeList = chatFrame.messageTypeList
    local infoPanel = chatFrame.infoPanel
    
    -- Retrieve Saved Variable Data Table
    local chatPath = self:GetSavedVariablePath()
    
    -- Save ChatFrame ID and Name
    local svData0 = {
        ["ID"] = id,
        ["Name"] = name,        
    }
    self:SVData(svData0, self.SVMode.SetKeyValue, chatPath)
    
    -- Save ChatFrame Font Information
    local fontPath = self:SVData("Fonts",self.SVMode.RetrieveSubTable,chatPath)
    local svData1 = {
        ["FontFile"] = fontFile,
        ["FontHeight"] = fontHeight,
        ["FontSize"] = fontSize,
        ["FontFlags"] = fontFlags,        
    }
    self:SVData(svData1, self.SVMode.SetKeyValue, fontPath)
    
    -- Save Chat Frame Messages
    chatPath["MessageTypes"] = {}
    local mtPath = self:SVData("MessageTypes",self.SVMode.RetrieveSubTable,chatPath)
    self:SVData(chatFrame.messageTypeList, self.SVMode.SetKeyValue, mtPath)

    -- Save Chat Frame Channels
    chatPath["Channels"] = {}
    local clPath = self:SVData("Channels",self.SVMode.RetrieveSubTable,chatPath)
    self:SVData(chatFrame.channelList, self.SVMode.SetKeyValue, clPath)
    
end

--[[ ]]--
function addonData:UseChatFrame(chatFrame)

    -- Tell us that we are using it in an infoPanel as that isn't docked or shown
    chatFrame.infoPanel = true
    self.chatFrame = chatFrame
    self.chatTab = _G["ChatFrame"..chatFrame:GetID().."Tab"]
    FCF_UnDockFrame(chatFrame)    
    
    -- Just in case there are remnant settings, remove them all
    ChatFrame_RemoveAllMessageGroups(chatFrame)
    ChatFrame_RemoveAllChannels(chatFrame)
    
    -- Load saved settings from the saved variables file
    -- And apply them to the chat frame being used
    local chatPath = self:GetSavedVariablePath()
    if chatPath["Name"] then
        FCF_SetWindowName(chatFrame, chatPath["Name"])
    end
    if chatPath["Fonts"] then
        chatFrame:SetFont(chatPath["Fonts"]["FontFile"],chatPath["Fonts"]["FontSize"],chatPath["Fonts"]["FontFlags"])
        FCF_SetChatWindowFontSize(nil, chatFrame, chatPath["Fonts"]["FontSize"] or chatPath["Fonts"]["FontHeight"] or 14)
    end
    if chatPath["MessageTypes"] then
        ChatFrame_RegisterForMessages(chatFrame, unpack(chatPath["MessageTypes"]))
    end
    
    if chatPath["Channels"] then
        ChatFrame_RegisterForChannels(chatFrame,unpack(chatPath["Channels"]))          
    end
    FCF_SetWindowColor(chatFrame, 0, 0, 0)
    FCF_SetWindowAlpha(chatFrame, 0)
end

--[[ ]]--
function addonData:ShowChatFrame(show)
    local chatFrame = self.chatFrame
    local chatTab = self.chatTab
    if show then
        chatFrame:Show()
        chatTab:Show()
    else
        chatFrame:Hide()
        chatTab:Hide()
    end
    SetChatWindowShown(chatFrame:GetID(), show)
end

--[[ ]]--
function addonData:RelocateFrame(plugin)
    local chatFrame = self.chatFrame
    local chatTab = self.chatTab
    
    MOVING_CHATFRAME = chatFrame
    FCF_SetLocked(chatFrame, false)
    FCF_UnDockFrame(chatFrame)
    
	chatFrame:StartMoving()
	chatFrame:StartSizing()
	chatFrame:SetParent(nil)
	chatFrame:SetParent( plugin.container )
	chatFrame:ClearAllPoints()    
	chatFrame:SetPoint( "TOPLEFT", plugin.container, "TOPLEFT", 5 * nUI.hScale, - 1 * (plugin.options.btn_size * nUI.vScale) )
	chatFrame:SetPoint( "BOTTOMRIGHT", plugin.container, "BOTTOMRIGHT", - 2 * (plugin.options.btn_size * nUI.hScale), 5 * nUI.vScale ) chatFrame:StopMovingOrSizing()
    
	chatTab:SetParent(nil)
	chatTab:SetParent( plugin.container )
    
    if chatFrame.Background then
        chatFrame.Background:ClearAllPoints()
        chatFrame.Background:SetAllPoints(chatFrame)
    end
    
    if chatFrame.ScrollBar then
        chatFrame.ScrollBar:ClearAllPoints()
        chatFrame.ScrollBar:SetPoint("TOPLEFT", chatFrame, "TOPRIGHT", 0, 0)
    end
    
    if chatFrame.ResizeButton then
        chatFrame.ResizeButton:ClearAllPoints()
        chatFrame.ResizeButton:SetPoint("BOTTOMRIGHT", chatFrame, "BOTTOMRIGHT", plugin.options.btn_size * nUI.hScale, 0)
    end
    
    if chatFrame.ScrollToBottomButton and chatFrame.ResizeButton then
        chatFrame.ScrollToBottomButton:ClearAllPoints();
        chatFrame.ScrollToBottomButton:SetPoint( "BOTTOM", chatFrame.ResizeButton, "TOP", 0, 0 );
    end

    if chatFrame.buttonFrame and chatFrame.ScrollBar then
        chatFrame.buttonFrame:ClearAllPoints();
        chatFrame.buttonFrame:SetPoint( "TOPLEFT", chatFrame.ScrollBar, "TOPRIGHT", plugin.options.btn_size * nUI.hScale + 1, 0 );
        chatFrame.buttonFrame:SetPoint( "BOTTOMLEFT", chatFrame.ScrollBar, "BOTTOMRIGHT", plugin.options.btn_size * nUI.hScale + 1, 0 );
        chatFrame.buttonFrame:EnableMouse( 0 );
    end
    
    FCF_SetTabPosition(chatFrame, 0)
    FCF_SetLocked(chatFrame, true)
    MOVING_CHATFRAME = nil    
    FCF_SetButtonSide(chatFrame, "right")
end

-- taken from FCF_ResetChatWindows in FloatingChatFrame.lua
function addonData:ResetChatFrame(chatFrame)
    chatFrame.isInitialized = 0
    FCF_SetTabPosition(chatFrame, 0)
    FCF_Close(chatFrame)
    FCF_UnDockFrame(chatFrame)
    FCF_SetWindowName(chatFrame, "")
    ChatFrame_RemoveAllMessageGroups(chatFrame)
    ChatFrame_RemoveAllChannels(chatFrame)
    ChatFrame_ReceiveAllPrivateMessages(chatFrame)
    FCF_SetChatWindowFontSize(nil, chatFrame, 14)
    FCF_SetWindowColor(chatFrame, DEFAULT_CHATFRAME_COLOR.r, DEFAULT_CHATFRAME_COLOR.g, DEFAULT_CHATFRAME_COLOR.b)
    FCF_SetWindowAlpha(chatFrame, DEFAULT_CHATFRAME_ALPHA)
end


-- Initialise the specifics of the addon 
function addonData:InitPlugin(plugin, enabled)
    -- Use to disable the infopanel if certain plugin specific requirements aren't met
    -- such as a class specific plugin or a profession specific plugin
    return enabled
end

-- Handle resizing of the window 
function addonData:ResizePlugin(plugin, scale, height, width)

    -- Make sure frames are fitted as needed regardless of changes to size
	local options  = plugin.options;
	local btn_size = options.btn_size * scale;

	plugin.height = height;
	plugin.width = width;
		
	-- If we haven't created the frame yet then no need to proceed quite yet
	if ( not self.chatFrame ) then return end
   
    local chatFrame = self.chatFrame
        
    -- set the size of the buttons in the chat frame and the chat frame itself
    if plugin.btn_size ~= btn_size then
        plugin.btn_size = btn_size;
        
        chatFrame:SetWidth( plugin.container:GetWidth() - (2 * btn_size) )
        chatFrame:SetHeight( plugin.container:GetHeight() - (1 * btn_size))

        if chatFrame.ScrollBar then
            chatFrame.ScrollBar:SetWidth(btn_size)
            chatFrame.ScrollBar:SetHeight(btn_size)
        end
        
        if chatFrame.ResizeButton then
            chatFrame.ResizeButton:SetWidth(btn_size * 0.5)
            chatFrame.ResizeButton:SetHeight(btn_size * 0.5)
        end
        
        if chatFrame.ScrollToBottomButton then
            chatFrame.ScrollToBottomButton:SetWidth(btn_size)
            chatFrame.ScrollToBottomButton:SetHeight(btn_size)
        end
        
        if chatFrame.buttonFrame then
            chatFrame.buttonFrame:SetWidth(btn_size)
            chatFrame.buttonFrame:SetHeight(btn_size)
        end

    end

end

--[[ React to enabling the plugin ]]
function addonData:EnablePlugin(plugin, enabled)
    local chatData = self:GetSavedVariablePath()
    local chatKey = self:FindAvailableChatFrame(chatData)
    if not chatKey then 
        return false
    else
        -- Rebuild the required frames
        -- This should show the container in the bottom left
        -- load settings last saved or create from basic
        local chatFrame = _G[chatKey]
        self:UseChatFrame(chatFrame)
        self:SaveChatFrameInfo(chatFrame)
        self:RelocateFrame(plugin)
        RegisterStateDriver( chatFrame.buttonFrame, "visibility", "hide" )
        RegisterStateDriver( chatFrame.ResizeButton, "visibility", "hide" )
        RegisterStateDriver( chatFrame.clickAnywhereButton, "visibility", "hide" )        
        self:ShowChatFrame(1)                
        return true
    end
end


--[[ React to disabling the plugin ]]
function addonData:DisablePlugin(plugin, enabled)
    local chatFrame = self.chatFrame
    if chatFrame then
        self:SaveChatFrameInfo(chatFrame)
        chatFrame.infoPanel = false        
        self:ResetChatFrame(chatFrame)
    end
end

--[[ React to selecting the plugin - showing it ]]
function addonData:SelectPlugin(plugin,selected)
    -- Specific changes that selected plugins need to have if any
end

--[[ React to deselecting the plugin hiding it ]]
function addonData:DeSelectPlugin(plugin, selected)
    -- Specific changes that deselected plugins need to have if any
end

-- Handle any plugin specific events here
function addonData:OnEvent(plugin,event,arg1,arg2,arg3)
    if event == "UPDATE_FLOATING_CHAT_WINDOWS" or event == "UPDATE_CHAT_WINDOWS" then
        self.chatWindowsReady = true
        
    elseif event == "PLAYER_LOGOUT" then
        if self.chatFrame then
            self:DisablePlugin(plugin,false)
        end
        
    end
end

-- Register the plugin specific events here
function addonData:RegisterEvents(plugin)
    plugin:RegisterEvent("UPDATE_FLOATING_CHAT_WINDOWS") -- Only on Reload ?
    plugin:RegisterEvent("UPDATE_CHAT_WINDOWS") -- Always on startup/reload with last chatframe (Chat 10) the current frame 
    plugin:RegisterEvent("PLAYER_LOGOUT") -- Always Disable plugin on logout
end

