local addonName, addonData = ...

-- Access Mode for the Saved Variable Access Function
addonData.SVMode = {
    Add = 1,
    Remove = 2,
    RetrieveKey = 3,
    RetrieveValue = 4,
    SetKeyValue = 5,
    RetrieveKeyValue = 6,
    RetrieveSubTable = 7,
}

addonData.SVType = {
    Account = 1,
    Character = 2,
}

addonData.SVKey = {
    Character = 1,
    None = 2,
}

--[[ Make sure we have the right table for the logged in character or for all characters on all realms ]]
function addonData:GetPlayerKey()
    local name = GetUnitName("player")
    local realm = GetRealmName()
    return name, realm
end

--[[ Work with the Saved Variable tables ]]
function addonData:SVAction(item,mode,svData)
    local retData = {}
    
    -- Retrieve either the key (k) or value (v) which will be returned for the plugin to operate on
    if mode == self.SVMode.RetrieveKey or mode == self.SVMode.RetrieveValue then
        local keyMode = (mode == self.SVMode.RetrieveKey)
        local valueMode = (mode == self.SVMode.RetrieveValue)
        for k,v in pairs(svData) do
            table.insert(retData,keyMode and i or v)
        end    
        
        
    else
        
        -- These tests need the item so move on if this doesn't apply
        if not item then return end  
        
        -- Add the requested item to the requested table 
        if mode == self.SVMode.Add then
            svData[item] = true
            
        -- Remove the requested item from the requested table
        elseif mode == self.SVMode.Remove then
            svData[item] = nil

        -- Deal with keyvalue table values
        elseif mode == self.SVMode.SetKeyValue then
            for k,v in pairs(item) do
                svData[k] = v
            end
            
        elseif mode == self.SVMode.RetrieveKeyValue then
            for k,v in pairs(svData) do
                retData[k] = v
            end
            
        elseif mode == self.SVMode.RetrieveSubTable then
            svData[item] = svData[item] or {}
            retData = svData[item]
        end
            
    end
    return retData
end
