--[[ Use Addon Wide Data Table ]]--
local addonName,addon = ...

--[[ Create the InfoPanel Plugin ]]--
local plugin    = CreateFrame( "Frame", addon.PluginName, nUI_Dashboard.Anchor )
plugin.active   = true

-- Keep track of event loaded status
local OAL = false
local OVL = false

--[[ Handle the addons Events ]]--
local function onEvent(self,event,arg1,arg2,arg3)
    
    -- This file handles loading of the addon and variables
    if event == "ADDON_LOADED" or event == "VARIABLES_LOADED" then
        
        if ( event == "ADDON_LOADED" ) then
            if ( arg1 == addonName ) then
                OAL = true
                self:UnregisterEvent(event)
            end

        elseif event == "VARIABLES_LOADED" then
            OVL = true
            self:UnregisterEvent(event)
        end	

        -- If both the addon and the saved variables are loaded then we can continue
        -- NOTE I am not sure if system and/or addon specific saved variables so erring on side of caution 
        -- This is where you should get your plugin to register any other events it wants to know about
        if OAL and OVL then
            addon:OnAddonLoaded(plugin)	
        end
                        
    -- Pass all other events to the plugin
    else
        addon:OnEvent(self,event,arg1,arg2,arg3)
        
    end
end

--[[ Set Up the Event Script ]]--
plugin:SetScript( "OnEvent", onEvent )
plugin:RegisterEvent( "ADDON_LOADED" )
plugin:RegisterEvent( "VARIABLES_LOADED" )


--[[ The InfoPanel resizing routine ]]--
plugin.sizeChanged = function( scale, height, width )
	plugin.scale = addon:OnInfoPanelResize(plugin, scale, height, width)
end	

--[[ The InfoPanel enabling routine ]]--
plugin.setEnabled = function( enabled )
    plugin.enabled = addon:OnInfoPanelEnable(enabled, plugin)
end

--[[ The InfoPanel selection routine ]]--
plugin.setSelected = function( selected )
    plugin.selected = addon:OnInfoPanelSelect(selected, plugin)
end

--[[ The InfoPanel initialisation routine ]]--
plugin.initPanel = function( container, options )
	plugin.container = container
	plugin.options   = options
    if options and options.enabled then
	   plugin.setEnabled( addon:OnInfoPanelInit(plugin, options.enabled) )
    end
end
